#include "stdafx.h"
#include "LayoutManager.h"


LayoutManager::LayoutManager()
{
	m_owner=NULL;
	m_origSizeInitialized=false;
}

void LayoutManager::setOrigSize(CRect *crect)
{
	m_origSize=crect;
	m_origSizeInitialized=true;
}

void LayoutManager::setOwner(CWnd *owner)
{
	m_owner=owner;
}

void LayoutManager::add(CWnd *element, float hgrow, float vgrow, int anchor)
{
	CRect crect;

	element->GetWindowRect(&crect);
	m_owner->ScreenToClient(&crect);

	LayoutRecord *rec=new LayoutRecord(element,hgrow,vgrow,anchor,crect);

	m_elements.push_back(rec);
}

void LayoutManager::OnSize(UINT nType, int dx, int dy)
{
	int widthChange,heightChange;

	widthChange=dx-m_origSize.Width();
	heightChange=dy-m_origSize.Height();

	std::list<LayoutRecord*>::iterator it;

    for (it=m_elements.begin(); it!=m_elements.end(); ++it) 
		{
		int newwidth, newheight, x, y;

		LayoutRecord *lr=*it;

		newwidth=lr->m_origPos.Width()+widthChange*lr->m_hgrow;
		newheight=lr->m_origPos.Height()+heightChange*lr->m_vgrow;

		switch (lr->m_anchor)
		{
		case kTopLeft:
			x=lr->m_origPos.left;
			y=lr->m_origPos.top;
			break;
		case kBottomLeft:
			x=lr->m_origPos.left;
			y=lr->m_origPos.bottom-newheight+heightChange;
			break;
		case kTopRight:
			x=lr->m_origPos.right-newwidth+widthChange;
			y=lr->m_origPos.top;
			break;
		case kBottomRight:
			x=lr->m_origPos.right-newwidth+widthChange;
			y=lr->m_origPos.bottom-newheight+heightChange;
		}

		lr->m_element->MoveWindow(x,y,newwidth,newheight);
		lr->m_element->Invalidate(false);
		}
	
}

void LayoutManager::OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI)
{
	if (!m_origSizeInitialized)
		return;

	lpMMI->ptMinTrackSize.x=m_origSize.Width();
	lpMMI->ptMinTrackSize.y=m_origSize.Height();
}
